###############################################################################
#                                                                             #
#                               Repeat After Me                               #
#                               _______________                               #
#                                                                             #
#                            Version: 2022-03-30                              #
#                       Created with: Godot 3.4.2                             #
#                             Author: Denis NICOLAS                           #
#                            License: GNU GPL Version 3                       #
#                                                                             #
###############################################################################

extends Spatial

var config = ConfigFile.new()


# Called when the node enters the scene tree for the first time.
func _ready():
	var err = config.load("user://config.cfg")

	# If the file didn't load, ignore it.
	if err == OK:
		# Load states and flip the toggles if necessary
		Var.messages = config.get_value("States", "messages", true)
		
		var no_repetition = config.get_value("States", "no_repetition", false)
		if no_repetition:
			toggle_no_repetition()
			
		var play_last_only = config.get_value("States", "play_last_only", false)
		if play_last_only:
			toggle_play_last_only()
		
	# Load the Hi-Score corresponding to the current state
	load_hi_score()
	
	# Scale the flower up
	var anim := $Center/AnimationPlayer as AnimationPlayer
	anim.play("Scale")
	yield(anim, "animation_finished")
	
	# Play the demo sequence
	play()


# Called every frame. 'delta' is the elapsed time since the previous frame.
#func _process(delta):
#	pass


func _on_Area_input_event(_camera, event: InputEvent, _position, _normal, _shape_idx, button_nb):
	if !Var.interactive or Var.animating:
		return
	if event is InputEventMouseButton:
		var ev := event as InputEventMouseButton
		if ev.button_index == BUTTON_LEFT and ev.pressed == true:
			# A petal has been clicked!
			Var.animating = true
			blink(button_nb)


func _on_Score_pressed():
	if Var.interactive and not Var.animating:
		start_game()


func load_hi_score():
	var err = config.load("user://config.cfg")

	# If the file didn't load, ignore it.
	if err != OK:
		return

	# Get the Hi-Score corresponding to the current state
	var val := config.get_value("Hi-Scores", str(Var.no_repetition) + str(Var.play_last_only), 0) as int
	set_hi_score(val)


func start_game():
	if Var.debug:
		print("start_game()")

	Var.game_over = false
	set_interactive(false)
	Var.sequence = []
	Var.pos = 0
	Var.available = [0, 1, 2, 3]

	randomize()

	# Reset hi-score color
	var hi_score := get_node("GUI/HiScore/VBoxC/Button") as Button
	hi_score.modulate = Color.white
	
	# Reset the score
	update_score()


func update_score():
	if Var.debug:
		print("update_score()")

	# Update the hi-score, if necessary
	var hi_score := get_node("GUI/HiScore/VBoxC/Button") as Button
	if Var.pos > int(hi_score.text):
		set_hi_score(Var.pos)
		
	var path := "GUI/Bottom/HBoxC/VBoxC/Score"
	
	# Fade-out the score
	var anim := get_node(path + "/AnimationPlayer") as AnimationPlayer
	anim.play("Fade-out")
	yield(anim, "animation_finished")
	
	# Update the score
	var score := get_node(path) as Button
	score.text = str(Var.pos)
	
	# Fade-in the score
	anim.play_backwards("Fade-out")
	yield(anim, "animation_finished")
	
	# Sequence gets longer
	draw()


func set_hi_score(val: int):
	var path := "GUI/HiScore/VBoxC/Button"
	
	# Fade-out the hi-score
	var anim := get_node(path + "/AnimationPlayer") as AnimationPlayer
	anim.play("Fade-out")
	yield(anim, "animation_finished")
	
	# Update the hi-score
	var hi_score := get_node(path) as Button
	if Var.pos > int(hi_score.text):
		# New hi-score will appear in green
		hi_score.modulate = Color(0, 1, 0, 0)
	hi_score.text = str(val)
	
	# Fade-in the hi-score
	anim.play_backwards("Fade-out")
	yield(anim, "animation_finished")
	
	# Store the value
	config.set_value("Hi-Scores", str(Var.no_repetition) + str(Var.play_last_only), val)
	
	# Save it to a file (overwrite if it already exists)
	config.save("user://config.cfg")


# Append one item in the sequence and play the sequence from pos.    
func draw():
	if Var.debug:
		print("draw()")
		
	var r := randi() % len(Var.available)

	Var.sequence.append(Var.available[r])
	if Var.no_repetition:
		# Make this item unavailable for the next draw
		Var.available.pop_at(r)
		if len(Var.available) < 3:
			# Make the previous item in the sequence available again for the next draw
			Var.available.append(Var.sequence[len(Var.sequence) - 2])
	
	if Var.play_last_only:
		# Play only the last item in the sequence
		Var.pos = len(Var.sequence) - 1
	else:
		# Play the whole sequence
		Var.pos = 0
	play()


# Blink the Button corresponding to pos in the sequence.
func play():
	if Var.debug:
		print("play()")
		
	var button_nb = Var.sequence[Var.pos]
	blink(button_nb)


func blink(button_nb: int):
	if Var.debug:
		print("blink(" + str(button_nb) + ")")

	var button_path := "Center/Petal_" + str(button_nb)

	var sound := get_node(button_path + "/AudioStreamPlayer2D") as AudioStreamPlayer2D
	sound.play()
	
	var anim := get_node(button_path + "/AnimationPlayer") as AnimationPlayer
	anim.play("Blink")
	yield(anim, "animation_finished")
	if Var.interactive:
		Var.animating = false
		if not Var.game_over:
			check(button_nb)
		else:
			Var.sequence.append(button_nb)
			var size := Var.sequence.size()
			if size > 3:
				if Var.sequence.slice(size - 4, size - 1) == [1, 2, 0, 3]:
					var easter_egg_dialog := get_node("/root/RAM/GUI/EasterEggDialog") as Popup
					easter_egg_dialog.popup_centered()
	else:
		play_next()


# Check that the button activated by the player
# corresponds to the button stored in the sequence.
func check(button):
	if Var.debug:
		print("check(" + str(button) + ")")
		
	if button == Var.sequence[Var.pos]:
		# Button clicked is correct!
		Var.pos = Var.pos + 1
		if Var.pos == len(Var.sequence):
			# Sequence successfully reproduced!
			set_interactive(false)
			update_score()
	else:
		# Player made a mistake!
		error()


# Called when the "Blink" animation is finished and interactive is false.
func play_next():
	if Var.debug:
		print("play_next()")
		
	yield(get_tree().create_timer(0.1), "timeout")
	
	Var.pos = Var.pos + 1
	if Var.pos < len(Var.sequence):
		# Blink the next Button in the sequence.
		play()
	else:
		# The complete sequence has been played.
		Var.pos = 0
		if Var.game_over:
			Var.sequence = []
			var hi_score := get_node("GUI/HiScore/VBoxC/Button") as Button
			if hi_score.modulate == Color.green:
				# Correction has been played after mistake, new Hi-Score.
				show_message("CONGRATULATIONS")
				var anim := get_node("Center/AnimationPlayer") as AnimationPlayer
				anim.play("Hi-Score")
				yield(anim, "animation_finished")
				hi_score.modulate = Color.white
		set_interactive(true)


func error():
	if Var.debug:
		print("error()")
		
	Var.game_over = true
	
	# Reset color of the core.
	var core := get_node("Center/Core") as MeshInstance
	core.material_override.albedo_color = Color(0.88, 0.88, 0.88)
	
	set_interactive(false)
	
	var path := "Center"

	var sound := get_node(path + "/AudioStreamPlayer") as AudioStreamPlayer
	sound.play()
	
	Input.vibrate_handheld()

	var anim := get_node(path + "/AnimationPlayer") as AnimationPlayer
	anim.play("Error")
	yield(anim, "animation_finished")

	# Play the complete sequence after a mistake from the user:
	Var.pos = 0
	play()


func set_interactive(val: bool):
	if Var.debug:
		print("set_interactive(" + str(val) + ")")
		
	Var.interactive = val
	
	if not Var.game_over:
		var core := get_node("Center/Core") as MeshInstance
		if Var.interactive:
			core.material_override.albedo_color = Color(0, 0.88, 0)
			show_message("PLAY_SEQUENCE_BACK")
		else:
			core.material_override.albedo_color = Color(0.88, 0, 0)
			if Var.play_last_only:
				show_message("MEMORIZE_APPENDED_ELEMENT")
			else:
				show_message("MEMORIZE_SEQUENCE")


func _on_Toggle_no_repetition_input_event(_camera, event: InputEvent, _position, _normal, _shape_idx):
	if !Var.game_over or !Var.interactive:
		return
	if event is InputEventMouseButton:
		var ev := event as InputEventMouseButton
		if ev.button_index == BUTTON_LEFT and ev.pressed == true:
			toggle_no_repetition()
			load_hi_score()


func toggle_no_repetition():
	Var.no_repetition = !Var.no_repetition

	config.set_value("States", "no_repetition", Var.no_repetition)
	config.save("user://config.cfg")
	
	var anim := get_node("Toggle_no_repetition/AnimationPlayer") as AnimationPlayer
	if Var.no_repetition:
		anim.play("Flip")
		show_message("NO_REPETITION")
	else:
		anim.play_backwards("Flip")
		show_message("REPETITION_ALLOWED")
	yield(anim, "animation_finished")


func _on_Toggle_play_last_only_input_event(_camera, event: InputEvent, _position, _normal, _shape_idx):
	if !Var.game_over or !Var.interactive:
		return
	if event is InputEventMouseButton:
		var ev := event as InputEventMouseButton
		if ev.button_index == BUTTON_LEFT and ev.pressed == true:
			toggle_play_last_only()
			load_hi_score()


func toggle_play_last_only():
	Var.play_last_only = !Var.play_last_only

	config.set_value("States", "play_last_only", Var.play_last_only)
	config.save("user://config.cfg")
	
	var anim := get_node("Toggle_play_last_only/AnimationPlayer") as AnimationPlayer
	if Var.play_last_only:
		anim.play("Flip")
		show_message("LAST_ELEMENT_ONLY")
	else:
		anim.play_backwards("Flip")
		show_message("WHOLE_SEQUENCE")
	yield(anim, "animation_finished")


func show_message(message: String):
	if Var.messages:
		var path := "GUI/Bottom/HBoxC/VBoxC/HBoxC/StatusBar"
		var status_bar = get_node(path)
		status_bar.display(message)
